@@ -1,4 +1,4 @@
-# Copyright 2019-2021 Cambridge Quantum Computing
+# Copyright 2019-2022 Cambridge Quantum Computing
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +13,17 @@
 # limitations under the License.
 
 import json
-from typing import cast, Iterable, List, Optional, Sequence, Union
+from typing import cast, Iterable, List, Optional, Sequence, Union, Any
 from uuid import uuid4
 from logging import warning
 
 import numpy as np
-from pyquil.api import QuantumComputer, WavefunctionSimulator
+from pyquil.api import (
+    QuantumComputer,
+    WavefunctionSimulator,
+    list_quantum_computers,
+    get_qc,
+)
 from pyquil.gates import I
 from pyquil.paulis import ID, PauliSum, PauliTerm
 from pyquil.quilatom import Qubit as Qubit_
@@ -108,23 +113,7 @@ class ForestBackend(Backend):
         """
         super().__init__()
         self._qc: QuantumComputer = qc
-
-        char_dict: dict = process_characterisation(self._qc)
-        arch = char_dict.get("Architecture", Architecture([]))
-        node_errors = char_dict.get("NodeErrors")
-        link_errors = char_dict.get("EdgeErrors")
-        averaged_errors = get_avg_characterisation(char_dict)
-        self._backend_info = BackendInfo(
-            type(self).__name__,
-            qc.name,
-            __extension_version__,
-            arch,
-            self._GATE_SET,
-            all_node_gate_errors=node_errors,
-            all_edge_gate_errors=link_errors,
-            averaged_node_gate_errors=averaged_errors["node_errors"],
-            averaged_edge_gate_errors=averaged_errors["link_errors"],
-        )
+        self._backend_info = self._get_backend_info(self._qc)
 
     @property
     def required_predicates(self) -> List[Predicate]:
@@ -136,6 +125,9 @@ class ForestBackend(Backend):
             ConnectivityPredicate(self.backend_info.architecture),
         ]
 
+    def rebase_pass(self) -> BasePass:
+        return RebaseQuil()
+
     def default_compilation_pass(self, optimisation_level: int = 1) -> BasePass:
         assert optimisation_level in range(3)
         passlist = [
@@ -162,7 +154,7 @@ class ForestBackend(Backend):
             passlist.append(CliffordSimp(False))
         if optimisation_level > 0:
             passlist.append(SynthesiseTket())
-        passlist.append(RebaseQuil())
+        passlist.append(self.rebase_pass())
         if optimisation_level > 0:
             passlist.extend(
                 [
@@ -276,6 +268,36 @@ class ForestBackend(Backend):
     def backend_info(self) -> BackendInfo:
         return self._backend_info
 
+    @classmethod
+    def _get_backend_info(cls, qc: QuantumComputer) -> BackendInfo:
+        char_dict: dict = process_characterisation(qc)
+        arch = char_dict.get("Architecture", Architecture([]))
+        node_errors = char_dict.get("NodeErrors")
+        link_errors = char_dict.get("EdgeErrors")
+        averaged_errors = get_avg_characterisation(char_dict)
+        return BackendInfo(
+            cls.__name__,
+            qc.name,
+            __extension_version__,
+            arch,
+            cls._GATE_SET,
+            all_node_gate_errors=node_errors,
+            all_edge_gate_errors=link_errors,
+            averaged_node_gate_errors=averaged_errors["node_errors"],
+            averaged_edge_gate_errors=averaged_errors["link_errors"],
+        )
+
+    @classmethod
+    def available_devices(cls, **kwargs: Any) -> List[BackendInfo]:
+        """
+        See :py:meth:`pytket.backends.Backend.available_devices`.
+        Supported kwargs: `qpus` (default true), `qvms` (default false).
+        """
+        if "qvms" not in kwargs:
+            kwargs["qvms"] = False
+        qc_name_list = list_quantum_computers(**kwargs)
+        return [cls._get_backend_info(get_qc(name)) for name in qc_name_list]
+
 
 class ForestStateBackend(Backend):
     _supports_state = True
@@ -316,6 +338,9 @@ class ForestStateBackend(Backend):
             DefaultRegisterPredicate(),
         ]
 
+    def rebase_pass(self) -> BasePass:
+        return RebaseQuil()
+
     def default_compilation_pass(self, optimisation_level: int = 1) -> BasePass:
         assert optimisation_level in range(3)
         passlist = [DecomposeBoxes(), FlattenRegisters()]
@@ -323,7 +348,7 @@ class ForestStateBackend(Backend):
             passlist.append(SynthesiseTket())
         elif optimisation_level == 2:
             passlist.append(FullPeepholeOptimise())
-        passlist.append(RebaseQuil())
+        passlist.append(self.rebase_pass())
         if optimisation_level > 0:
             passlist.append(EulerAngleReduction(OpType.Rx, OpType.Rz))
         return SequencePass(passlist)
